#include "Riostream.h"
#include "TDatime.h"
#include "TFile.h"
#include "TTree.h"
#include "TBranch.h"
#include "TClassTable.h"
#include "TSystem.h"

using namespace std;

//application header files
#include "Vehicle.h"

void createNewRootFile();
void readRootFile();

void main() 
{
    
  createNewRootFile();
  readRootFile();
}


/*-------------------------------------------------------------------*/
void createNewRootFile() 
{

  TFile rootFile("Vehicle.root", "RECREATE");
  
  
  /* Create a tree                         */
  /* The tree is attached to the root file */
  TTree tree("car","car");
  
  /* We are going to populate the tree with a bunch of Vehicles */
  Vehicle *ptr = NULL;
  
  tree.Branch("car","Vehicle",&ptr,32000,0);

  /* create a Vehicle */
  ptr = new Vehicle();
  ptr->parameter = 1;
  ptr->numberOfSamples = 6;
  
  
  
  int indexCar = 0;
  for (int indexRsp=0; indexRsp<3; indexRsp++) 
    {
      //create one rsp
      vector<Table> rsp;                    
      for (int indexSegment=0; indexSegment<3; indexSegment++) 
	{
	  //create one segment                          
	  Table segment;
	  for (int indexRow=0; indexRow<3; indexRow++) 
	    {
	      //create one row and attach it to the segment 
	      segment.v.push_back(1000*(indexCar+1) + 100*(indexRsp+1) + 10*(indexSegment+1) + (indexRow+1));
	    }//row
	  //attach the segment to the rsp
	  rsp.push_back(segment);                         
	}//segment
      //attach the rsp to the car
      ptr->v.push_back(rsp);
    }//rsp
  
  
  
  /* push the Vehicle into the tree */
  tree.Fill();
  
  
  /* deallocate the Vehicle */
  delete ptr;
  
  /* create a 2nd Vehicle */
  ptr = new Vehicle();
  ptr->parameter = 456;
  ptr->numberOfSamples = 6;
  
  indexCar = 1;
  for (int indexRsp=0; indexRsp<3; indexRsp++) 
    {
      //create one rsp
      vector<Table> rsp;                    
      for (int indexSegment=0; indexSegment<3; indexSegment++) 
	{
	  //create one segment                          
	  Table segment;
	  for (int indexRow=0; indexRow<3; indexRow++) 
	    {
	      //create one row and attach it to the segment 
	      segment.v.push_back(1000*(indexCar+1) + 100*(indexRsp+1) + 10*(indexSegment+1) + (indexRow+1));
	    }//row
	  //attach the segment to the rsp
	  rsp.push_back(segment);                         
	}//segment
      //attach the rsp to the car
      ptr->v.push_back(rsp);
    }//rsp
  
  
  
  /* push the 2nd Vehicle into the tree */
  tree.Fill();
  
  /* close the root file */
  rootFile.Write();
  rootFile.Close();
  
  /* deallocate the 2nd Vehicle */
  delete ptr;

}

/*-------------------------------------------------------------------*/
void readRootFile() 
{
  /* pointer to the root file */
  TFile *rootFile = NULL;
  
  /* pointer to the tree that is contained within the root file */
  TTree *tree = NULL;
  Vehicle *ptr = NULL;
  
  /* open the root file */
  rootFile = new TFile("Vehicle.root", "READ");
  
  tree = (TTree*)rootFile->Get("car");
  tree->SetBranchAddress("car",&(ptr));

  /* how many cars are stored in the branch? */
  Long64_t  numberOfCars = tree->GetEntries();
  cout << "number of cars: " << numberOfCars << endl;
  
  /* read every car that is in the branch of the tree */
  for (Long64_t i=0; i<numberOfCars; i++) 
    {
      tree->GetEntry(i);
            
      /* process the retrieved Vehicle */        
      cout << "Vehicle[" << i << "]:"
	   <<"parameter=" << ptr->parameter << " ";
      cout << endl;
      
      
      //ptr->v is a vector of RSPs
      for (int indexRsp=0; indexRsp<ptr->v.size(); indexRsp++) 
	{
	  //extract one rsp
	  vector<Table> rsp = (vector<Table>)ptr->v[indexRsp];
	  //rps is a vector of segments
	  for (int indexSegment=0; indexSegment<rsp.size(); indexSegment++) 
	    {
	      //extract one segment
	      Table segment = (Table)rsp[indexSegment];
	      //segment contains rows
	      for (int indexRow=0; indexRow<segment.v.size(); indexRow++) 
		{
		  //extract one row
		  int row = segment.v[indexRow];
		  cout << "vector[" << indexRsp << "][" << indexSegment << "][" << indexRow << "]=" << row << endl;
		}
	    }
	  
	}
      
      cout << endl;

      ptr->Clear();
      
    }

  if (rootFile!=NULL) 
    {
      rootFile->Close();
      delete rootFile;
    }
  rootFile=NULL;
  
}
